@@ -1,3 +1,4 @@
+/* $Id: CoinFactorization1.cpp 1277 2010-04-21 20:19:32Z forrest $ */
 // Copyright (C) 2002, International Business Machines
 // Corporation and others.  All Rights Reserved.
 #if defined(_MSC_VER)
@@ -30,7 +31,7 @@ CoinFactorization::CoinFactorization ( const CoinFactorization &other)
   gutsOfCopy(other);
 }
 /// The real work of constructors etc
-void CoinFactorization::gutsOfDestructor(int type)
+void CoinFactorization::gutsOfDestructor(int )
 {
   delete [] denseArea_;
   delete [] densePermute_;
@@ -143,7 +144,7 @@ void CoinFactorization::gutsOfInitialize(int type)
     densePermute_=NULL;
     numberDense_=0;
     if (!persistenceFlag_) {
-      workArea_=CoinDoubleArrayWithLength();
+      workArea_=CoinFactorizationDoubleArrayWithLength();
       workArea2_=CoinUnsignedIntArrayWithLength();
       pivotColumn_=CoinIntArrayWithLength();
     }
@@ -285,7 +286,7 @@ int CoinFactorization::factorize (
   numberElements=0;
   int * indexColumnU = indexColumnU_.array();
   int * indexRowU = indexRowU_.array();
-  double * elementU = elementU_.array();
+  CoinFactorizationDouble * elementU = elementU_.array();
   for (i=0;i<numberRows;i++) {
     if (rowIsBasic[i]>=0) {
       indexRowU[numberElements]=i;
@@ -313,7 +314,7 @@ int CoinFactorization::factorize (
   numberBasic=0;
   if (status_ == 0) {
     int * permuteBack = permuteBack_.array();
-    int * back = pivotColumnBack_.array();
+    int * back = pivotColumnBack();
     for (i=0;i<numberRows;i++) {
       if (rowIsBasic[i]>=0) {
 	rowIsBasic[i]=permuteBack[back[numberBasic++]];
@@ -328,7 +329,7 @@ int CoinFactorization::factorize (
     // these arrays start off as copies of permute
     // (and we could use permute_ instead of pivotColumn (not back though))
     CoinMemcpyN ( permute_.array(), numberRows_ , pivotColumn_.array()  );
-    CoinMemcpyN ( permuteBack_.array(), numberRows_ , pivotColumnBack_.array()  );
+    CoinMemcpyN ( permuteBack_.array(), numberRows_ , pivotColumnBack()  );
   } else if (status_ == -1) {
     const int * pivotColumn = pivotColumn_.array();
     // mark as basic or non basic
@@ -375,16 +376,18 @@ int CoinFactorization::factorize (
   //copy
   CoinMemcpyN ( indicesRow, numberOfElements, indexRowU_.array() );
   CoinMemcpyN ( indicesColumn, numberOfElements, indexColumnU_.array() );
-  CoinMemcpyN ( elements, numberOfElements, elementU_.array() );
+  int i;
+  CoinFactorizationDouble * elementU = elementU_.array();
+  for (i=0;i<numberOfElements;i++)
+    elementU[i] = elements[i];
   lengthU_ = numberOfElements;
   maximumU_ = numberOfElements;
   preProcess ( 0 );
   factor (  );
   //say which column is pivoting on which row
-  int i;
   if (status_ == 0) {
     int * permuteBack = permuteBack_.array();
-    int * back = pivotColumnBack_.array();
+    int * back = pivotColumnBack();
     // permute so slacks on own rows etc
     for (i=0;i<numberOfColumns;i++) {
       permutation[i]=permuteBack[back[i]];
@@ -393,7 +396,7 @@ int CoinFactorization::factorize (
     // these arrays start off as copies of permute
     // (and we could use permute_ instead of pivotColumn (not back though))
     CoinMemcpyN ( permute_.array(), numberRows_ , pivotColumn_.array()  );
-    CoinMemcpyN ( permuteBack_.array(), numberRows_ , pivotColumnBack_.array()  );
+    CoinMemcpyN ( permuteBack_.array(), numberRows_ , pivotColumnBack()  );
   } else if (status_ == -1) {
     const int * pivotColumn = pivotColumn_.array();
     // mark as basic or non basic
@@ -416,11 +419,11 @@ int CoinFactorization::factorize (
    returns 0 -okay, -99 memory */
 int 
 CoinFactorization::factorizePart1 ( int numberOfRows,
-				   int numberOfColumns,
+				   int ,
 				   CoinBigIndex numberOfElements,
 				   int * indicesRow[],
 				   int * indicesColumn[],
-				   double * elements[],
+				   CoinFactorizationDouble * elements[],
 				   double areaFactor)
 {
   // maybe for speed will be better to leave as many regions as possible
@@ -454,7 +457,7 @@ CoinFactorization::factorizePart2 (int permutation[],int exactNumberElements)
   //say which column is pivoting on which row
   int i;
   int * permuteBack = permuteBack_.array();
-  int * back = pivotColumnBack_.array();
+  int * back = pivotColumnBack();
   // permute so slacks on own rows etc
   for (i=0;i<numberColumns_;i++) {
     permutation[i]=permuteBack[back[i]];
@@ -464,7 +467,7 @@ CoinFactorization::factorizePart2 (int permutation[],int exactNumberElements)
     // these arrays start off as copies of permute
     // (and we could use permute_ instead of pivotColumn (not back though))
     CoinMemcpyN ( permute_.array(), numberRows_ , pivotColumn_.array()  );
-    CoinMemcpyN ( permuteBack_.array(), numberRows_ , pivotColumnBack_.array()  );
+    CoinMemcpyN ( permuteBack_.array(), numberRows_ , pivotColumnBack()  );
   } else if (status_ == -1) {
     const int * pivotColumn = pivotColumn_.array();
     // mark as basic or non basic
@@ -495,7 +498,7 @@ CoinFactorization::show_self (  ) const
   const int * pivotColumn = pivotColumn_.array();
   for ( i = 0; i < numberRows_; i++ ) {
     std::cout << "r " << i << " " << pivotColumn[i];
-    if (pivotColumnBack_.array()) std::cout<< " " << pivotColumnBack_.array()[i];
+    if (pivotColumnBack()) std::cout<< " " << pivotColumnBack()[i];
     std::cout<< " " << permute_.array()[i];
     if (permuteBack_.array()) std::cout<< " " << permuteBack_.array()[i];
     std::cout<< " " << pivotRegion_.array()[i];
@@ -554,7 +557,7 @@ CoinFactorization::getAreas ( int numberOfRows,
 			 CoinBigIndex maximumL,
 			 CoinBigIndex maximumU )
 {
-
+  
   numberRows_ = numberOfRows;
   numberColumns_ = numberOfColumns;
   maximumRowsExtra_ = numberRows_ + maximumPivots_;
@@ -569,8 +572,8 @@ CoinFactorization::getAreas ( int numberOfRows,
   if ( areaFactor_ != 1.0 ) {
     if ((messageLevel_&16)!=0) 
       printf("Increasing factorization areas by %g\n",areaFactor_);
-    lengthAreaU_ =  (CoinBigIndex) (areaFactor_*lengthAreaU_);
-    lengthAreaL_ =  (CoinBigIndex) (areaFactor_*lengthAreaL_);
+    lengthAreaU_ =  static_cast<CoinBigIndex> (areaFactor_*lengthAreaU_);
+    lengthAreaL_ =  static_cast<CoinBigIndex> (areaFactor_*lengthAreaL_);
   }
   elementU_.conditionalNew( lengthAreaU_ );
   indexRowU_.conditionalNew( lengthAreaU_ );
@@ -603,10 +606,10 @@ CoinFactorization::getAreas ( int numberOfRows,
   permute_.conditionalNew( maximumRowsExtra_ + 1 );
   pivotRegion_.conditionalNew( maximumRowsExtra_ + 1 );
 #ifdef ZEROFAULT
-  memset(elementU_.array(),'a',lengthAreaU_*sizeof(double));
+  memset(elementU_.array(),'a',lengthAreaU_*sizeof(CoinFactorizationDouble));
   memset(indexRowU_.array(),'b',lengthAreaU_*sizeof(int));
   memset(indexColumnU_.array(),'c',lengthAreaU_*sizeof(int));
-  memset(elementL_.array(),'d',lengthAreaL_*sizeof(double));
+  memset(elementL_.array(),'d',lengthAreaL_*sizeof(CoinFactorizationDouble));
   memset(indexRowL_.array(),'e',lengthAreaL_*sizeof(int));
   memset(startColumnL_.array()+1,'f',numberRows_*sizeof(CoinBigIndex));
   memset(startRowU_.array(),'g',maximumRowsExtra_*sizeof(CoinBigIndex));
@@ -616,7 +619,7 @@ CoinFactorization::getAreas ( int numberOfRows,
   memset(nextRow_.array(),'k',(maximumRowsExtra_+1)*sizeof(int));
   memset(lastRow_.array(),'l',(maximumRowsExtra_+1)*sizeof(int));
   memset(permute_.array(),'l',(maximumRowsExtra_+1)*sizeof(int));
-  memset(pivotRegion_.array(),'m',(maximumRowsExtra_+1)*sizeof(double));
+  memset(pivotRegion_.array(),'m',(maximumRowsExtra_+1)*sizeof(CoinFactorizationDouble));
 #endif
   startColumnU_.conditionalNew( maximumColumnsExtra_ + 1 );
   numberInColumn_.conditionalNew( maximumColumnsExtra_ + 1 );
@@ -639,7 +642,7 @@ CoinFactorization::getAreas ( int numberOfRows,
     } else {
       biggerDimension_ = numberColumns_;
     }
-    firstCount_.conditionalNew( biggerDimension_ + 2 );
+    firstCount_.conditionalNew( CoinMax(biggerDimension_ + 2, maximumRowsExtra_+1) );
     nextCount_.conditionalNew( numberRows_ + numberColumns_ );
     lastCount_.conditionalNew( numberRows_ + numberColumns_ );
 #ifdef ZEROFAULT
@@ -662,11 +665,11 @@ CoinFactorization::getAreas ( int numberOfRows,
 //state is 0 - triplets, 1 - some counts etc , 2 - ..
 void
 CoinFactorization::preProcess ( int state,
-			   int possibleDuplicates )
+			   int )
 {
   int *indexRow = indexRowU_.array();
   int *indexColumn = indexColumnU_.array();
-  double *element = elementU_.array();
+  CoinFactorizationDouble *element = elementU_.array();
   CoinBigIndex numberElements = lengthU_;
   int *numberInRow = numberInRow_.array();
   int *numberInColumn = numberInColumn_.array();
@@ -709,7 +712,7 @@ CoinFactorization::preProcess ( int state,
 	int iColumn = indexColumn[k];
 
 	if ( iColumn >= 0 ) {
-	  double value = element[k];
+	  CoinFactorizationDouble value = element[k];
 	  int iRow = indexRow[k];
 
 	  indexColumn[k] = -1;
@@ -717,7 +720,7 @@ CoinFactorization::preProcess ( int state,
 	    CoinBigIndex iLook = startColumn[iColumn] - 1;
 
 	    startColumn[iColumn] = iLook;
-	    double valueSave = element[iLook];
+	    CoinFactorizationDouble valueSave = element[iLook];
 	    int iColumnSave = indexColumn[iLook];
 	    int iRowSave = indexRow[iLook];
 
@@ -755,7 +758,7 @@ CoinFactorization::preProcess ( int state,
 	  CoinBigIndex first = startColumn[iColumn];
 	  CoinBigIndex largest = first;
 	  int iRowSave = indexRow[first];
-	  double valueSave = element[first];
+	  CoinFactorizationDouble valueSave = element[first];
 	  double valueLargest = fabs ( valueSave );
 	  int iLook = numberInRow[iRowSave];
 
@@ -767,7 +770,7 @@ CoinFactorization::preProcess ( int state,
 
 	    numberInRow[iRow] = iLook + 1;
 	    indexColumn[startRow[iRow] + iLook] = iColumn;
-	    double value = element[k];
+	    CoinFactorizationDouble value = element[k];
 	    double valueAbs = fabs ( value );
 
 	    if ( valueAbs > valueLargest ) {
@@ -784,9 +787,6 @@ CoinFactorization::preProcess ( int state,
     }
   case 3:			//links and initialize pivots
     {
-      //set markRow so no rows updated
-      int * markRow = markRow_.array();
-      CoinFillN ( markRow, numberRows_, -1 );
       int *lastRow = lastRow_.array();
       int *nextRow = nextRow_.array();
       int *lastColumn = lastColumn_.array();
@@ -827,7 +827,7 @@ CoinFactorization::preProcess ( int state,
   case 4:			//move largest in column to beginning
     {
       CoinBigIndex i, k;
-      double * pivotRegion = pivotRegion_.array();
+      CoinFactorizationDouble * pivotRegion = pivotRegion_.array();
       int iColumn;
       int iRow;
       for ( iRow = 0; iRow < numberRows; iRow++ ) {
@@ -854,7 +854,8 @@ CoinFactorization::preProcess ( int state,
 	  CoinBigIndex end = first+number;
 	  for (  ; k < end; k++ ) {
 	    int iRow = indexRow[k];
-	    double value = element[k];
+	    assert (iRow<numberRows_);
+	    CoinFactorizationDouble value = element[k];
 	    if (numberInRow[iRow]>=0) {
 	      numberInRow[iRow]++;
 	      double valueAbs = fabs ( value );
@@ -913,10 +914,10 @@ CoinFactorization::preProcess ( int state,
     // modified 3
     {
       //set markRow so no rows updated
-      CoinFillN ( markRow_.array(), numberRows_, -1 );
+      //CoinFillN ( markRow_.array(), numberRows_, -1 );
       int *lastColumn = lastColumn_.array();
       int *nextColumn = nextColumn_.array();
-      double * pivotRegion = pivotRegion_.array();
+      CoinFactorizationDouble * pivotRegion = pivotRegion_.array();
 
       int iRow;
       int numberGood=0;
@@ -959,9 +960,28 @@ CoinFactorization::factor (  )
     totalElements_ = 0;
     {
       int * pivotColumn = pivotColumn_.array();
-      int * pivotRowL = pivotRowL_.array();
       if ( numberGoodU_ < numberRows_ ) {
 	int i, k;
+	// Clean out unset nextRow
+	int * nextRow = nextRow_.array();
+	//int nSing =0;
+	k=nextRow[maximumRowsExtra_];
+	while (k!=maximumRowsExtra_) {
+	  int iRow = k;
+	  k=nextRow[k];
+	  //nSing++;
+	  nextRow[iRow]=-1;
+	}
+	//assert (nSing);
+	//printf("%d singularities - good %d rows %d\n",nSing,
+	//     numberGoodU_,numberRows_);
+	// Now nextRow has -1 or sequence into numberGoodU_;
+	int * permuteA = permute_.array();
+	for ( i = 0; i < numberRows_; i++ ) {
+	  int iGood= nextRow[i];
+	  if (iGood>=0)
+	    permuteA[iGood]=i;
+	}
 
 	// swap arrays
 	permute_.swap(nextRow_);
@@ -973,7 +993,7 @@ CoinFactorization::factor (  )
 	  lastColumn[i] = -1;
 	}
 	for ( i = 0; i < numberGoodU_; i++ ) {
-	  int goodRow = pivotRowL[i];	//valid pivot row
+	  int goodRow = permuteA[i];	//valid pivot row
 	  int goodColumn = pivotColumn[i];
 
 	  lastRow[goodRow] = goodColumn;	//will now have -1 or column sequence
@@ -1059,18 +1079,16 @@ CoinFactorization::pivotRowSingleton ( int pivotRow,
       std::cout << "more memory needed in middle of invert" << std::endl;
     return false;
   }
-  int * pivotRowL = pivotRowL_.array();
-  pivotRowL[numberGoodL_] = pivotRow;
   CoinBigIndex * startColumnL = startColumnL_.array();
-  double * elementL = elementL_.array();
+  CoinFactorizationDouble * elementL = elementL_.array();
   int * indexRowL = indexRowL_.array();
   startColumnL[numberGoodL_] = l;	//for luck and first time
   numberGoodL_++;
   startColumnL[numberGoodL_] = l + numberDoColumn;
   lengthL_ += numberDoColumn;
-  double *elementU = elementU_.array();
-  double pivotElement = elementU[pivotRowPosition];
-  double pivotMultiplier = 1.0 / pivotElement;
+  CoinFactorizationDouble *elementU = elementU_.array();
+  CoinFactorizationDouble pivotElement = elementU[pivotRowPosition];
+  CoinFactorizationDouble pivotMultiplier = 1.0 / pivotElement;
 
   pivotRegion_.array()[numberGoodU_] = pivotMultiplier;
   CoinBigIndex i;
@@ -1173,9 +1191,10 @@ CoinFactorization::pivotColumnSingleton ( int pivotRow,
   nextRow[last] = next;
   lastRow[next] = last;
   nextRow[pivotRow] = numberGoodU_;	//use for permute
+  lastRow[pivotRow] =-2; //mark
   //clean up counts
-  double *elementU = elementU_.array();
-  double pivotElement = elementU[startColumn];
+  CoinFactorizationDouble *elementU = elementU_.array();
+  CoinFactorizationDouble pivotElement = elementU[startColumn];
 
   pivotRegion_.array()[numberGoodU_] = 1.0 / pivotElement;
   numberInColumn[pivotColumn] = 0;
@@ -1204,7 +1223,7 @@ CoinFactorization::pivotColumnSingleton ( int pivotRow,
                 numberInColumn[iColumn]);
 	if ( pivot != start ) {
 	  //move largest one up
-	  double value = elementU[start];
+	  CoinFactorizationDouble value = elementU[start];
 
 	  iRow = indexRowU[start];
 	  elementU[start] = elementU[pivot];
@@ -1216,14 +1235,14 @@ CoinFactorization::pivotColumnSingleton ( int pivotRow,
 	} else {
 	  //find new largest element
 	  int iRowSave = indexRowU[start + 1];
-	  double valueSave = elementU[start + 1];
+	  CoinFactorizationDouble valueSave = elementU[start + 1];
 	  double valueLargest = fabs ( valueSave );
 	  CoinBigIndex end = start + numberInColumn[iColumn];
 	  CoinBigIndex largest = start + 1;
 
 	  CoinBigIndex k;
 	  for ( k = start + 2; k < end; k++ ) {
-	    double value = elementU[k];
+	    CoinFactorizationDouble value = elementU[k];
 	    double valueAbs = fabs ( value );
 
 	    if ( valueAbs > valueLargest ) {
@@ -1251,7 +1270,6 @@ CoinFactorization::pivotColumnSingleton ( int pivotRow,
   //put in dummy pivot in L
   CoinBigIndex l = lengthL_;
 
-  pivotRowL_.array()[numberGoodL_] = pivotRow;
   CoinBigIndex * startColumnL = startColumnL_.array();
   startColumnL[numberGoodL_] = l;	//for luck and first time
   numberGoodL_++;
@@ -1275,10 +1293,10 @@ CoinFactorization::getColumnSpace ( int iColumn,
     numberInColumn[iColumn];
   CoinBigIndex * startColumnU = startColumnU_.array();
   CoinBigIndex space = lengthAreaU_ - startColumnU[maximumColumnsExtra_];
-  double *elementU = elementU_.array();
+  CoinFactorizationDouble *elementU = elementU_.array();
   int * indexRowU = indexRowU_.array();
 
-  if ( space < extraNeeded + number + 2 ) {
+  if ( space < extraNeeded + number + 4 ) {
     //compression
     int iColumn = nextColumn[maximumColumnsExtra_];
     CoinBigIndex put = 0;
@@ -1341,7 +1359,7 @@ CoinFactorization::getColumnSpace ( int iColumn,
     startColumnU[iColumn] = put + numberInColumnPlus[iColumn];
     if ( number < 50 ) {
       int *indexRow = indexRowU;
-      double *element = elementU;
+      CoinFactorizationDouble *element = elementU;
       int i = 0;
 
       if ( ( number & 1 ) != 0 ) {
@@ -1350,8 +1368,8 @@ CoinFactorization::getColumnSpace ( int iColumn,
 	i = 1;
       }
       for ( ; i < number; i += 2 ) {
-	double value0 = element[get + i];
-	double value1 = element[get + i + 1];
+	CoinFactorizationDouble value0 = element[get + i];
+	CoinFactorizationDouble value1 = element[get + i + 1];
 	int index0 = indexRow[get + i];
 	int index1 = indexRow[get + i + 1];
 
@@ -1366,8 +1384,17 @@ CoinFactorization::getColumnSpace ( int iColumn,
     }
     put += number;
     get += number;
-    //add 4 for luck
-    startColumnU[maximumColumnsExtra_] = put + extraNeeded + 4;
+    //add 2 for luck
+    startColumnU[maximumColumnsExtra_] = put + extraNeeded + 2;
+    if (startColumnU[maximumColumnsExtra_]>lengthAreaU_) {
+      // get more memory
+#ifdef CLP_DEVELOP
+      printf("put %d, needed %d, start %d, length %d\n",
+	     put,extraNeeded,startColumnU[maximumColumnsExtra_],
+	     lengthAreaU_);
+#endif
+      return false;
+    }
   } else {
     //take off space
     startColumnU[maximumColumnsExtra_] = startColumnU[last] +
@@ -1448,18 +1475,109 @@ CoinFactorization::getRowSpace ( int iRow,
   return true;
 }
 
+#if COIN_ONE_ETA_COPY
+/* Reorders U so contiguous and in order (if there is space)
+   Returns true if it could */
+bool 
+CoinFactorization::reorderU()
+{
+#if 1
+  return false;
+#else
+  if (numberRows_!=numberColumns_)
+    return false;
+  CoinBigIndex * startColumnU = startColumnU_.array();
+  int * numberInColumn = numberInColumn_.array();
+  int * numberInColumnPlus = numberInColumnPlus_.array();
+  int iColumn;
+  CoinBigIndex put = 0;
+  for (iColumn =0;iColumn<numberRows_;iColumn++) 
+    put += numberInColumnPlus[iColumn];
+  CoinBigIndex space = lengthAreaU_ - startColumnU[maximumColumnsExtra_];
+  if (space<put) {
+    //printf("Space %d out of %d - needed %d\n",
+    //   space,lengthAreaU_,put);
+    return false;
+  }
+  int *indexRowU = indexRowU_.array();
+  CoinFactorizationDouble *elementU = elementU_.array();
+  int * pivotColumn = pivotColumn_.array();
+  put = startColumnU[maximumColumnsExtra_];
+  for (int jColumn =0;jColumn<numberRows_;jColumn++) {
+    iColumn = pivotColumn[jColumn];
+    int n = numberInColumnPlus[iColumn];
+    CoinBigIndex getEnd = startColumnU[iColumn];
+    CoinBigIndex get = getEnd - n;
+    startColumnU[iColumn] = put;
+    numberInColumn[jColumn]=n;
+    CoinBigIndex i;
+    for ( i = get; i < getEnd; i++ ) {
+      indexRowU[put] = indexRowU[i];
+      elementU[put] = elementU[i];
+      put++;
+    }
+  }
+  // and pack down
+  put = 0;
+  for (int jColumn =0;jColumn<numberRows_;jColumn++) {
+    iColumn = pivotColumn[jColumn];
+    int n = numberInColumn[jColumn];
+    CoinBigIndex get = startColumnU[iColumn];
+    CoinBigIndex getEnd = get+n;
+    CoinBigIndex i;
+    for ( i = get; i < getEnd; i++ ) {
+      indexRowU[put] = indexRowU[i];
+      elementU[put] = elementU[i];
+      put++;
+    }
+  }
+  put=0;
+  for (iColumn =0;iColumn<numberRows_;iColumn++) { 
+    int n = numberInColumn[iColumn];
+    startColumnU[iColumn]=put;
+    put += n;
+    //numberInColumnPlus[iColumn]=n;
+    //numberInColumn[iColumn]=0; // necessary?
+    //pivotColumn[iColumn]=iColumn;
+  }
+#if 0
+  // reset nextColumn - probably not necessary
+  int * nextColumn = nextColumn_.array();
+  nextColumn[maximumColumnsExtra_]=0;
+  nextColumn[numberRows_-1] = maximumColumnsExtra_;
+  for (iColumn=0;iColumn<numberRows_-1;iColumn++)
+    nextColumn[iColumn]=iColumn+1;
+  // swap arrays
+  numberInColumn_.swap(numberInColumnPlus_);
+#endif
+  //return false;
+  return true;
+#endif
+}
+#endif
 //  cleanup.  End of factorization
 void
 CoinFactorization::cleanup (  )
 {
+#if COIN_ONE_ETA_COPY
+  bool compressDone = reorderU();
+  if (!compressDone) {
+    getColumnSpace ( 0, COIN_INT_MAX >> 1 );	//compress
+    // swap arrays
+    numberInColumn_.swap(numberInColumnPlus_);
+  }
+#else
   getColumnSpace ( 0, COIN_INT_MAX >> 1 );	//compress
+  // swap arrays
+    numberInColumn_.swap(numberInColumnPlus_);
+#endif
   CoinBigIndex * startColumnU = startColumnU_.array();
   CoinBigIndex lastU = startColumnU[maximumColumnsExtra_];
 
   //free some memory here
   saveColumn_.conditionalDelete();
   markRow_.conditionalDelete() ;
-  firstCount_.conditionalDelete() ;
+  //firstCount_.conditionalDelete() ;
   nextCount_.conditionalDelete() ;
   lastCount_.conditionalDelete() ;
   int * numberInRow = numberInRow_.array();
@@ -1471,6 +1589,31 @@ CoinFactorization::cleanup (  )
   //use nextRow for permutation  (as that is what it is)
   int i;
 
+#ifndef NDEBUG
+  {
+    if (numberGoodU_<numberRows_)
+      abort();
+    char * mark = new char[numberRows_];
+    memset(mark,0,numberRows_);
+    int * array;
+    array = nextRow_.array();
+    for ( i = 0; i < numberRows_; i++ ) {
+      int k = array[i];
+      if(k<0||k>=numberRows_)
+	printf("Bad a %d %d\n",i,k);
+      assert(k>=0&&k<numberRows_);
+      if(mark[k]==1)
+	printf("Bad a %d %d\n",i,k);
+      mark[k]=1;
+    }
+    for ( i = 0; i < numberRows_; i++ ) {
+      assert(mark[i]==1);
+      if(mark[i]!=1)
+	printf("Bad b %d\n",i);
+    }
+    delete [] mark;
+  }
+#endif
   // swap arrays
   permute_.swap(nextRow_);
   //safety feature
@@ -1488,42 +1631,45 @@ CoinFactorization::cleanup (  )
   }
   //redo nextRow_
 
-  // Redo total elements
-  totalElements_=0;
-  for ( i = 0; i < numberColumns_; i++ ) {
-    int number = numberInColumn[i];	//always 0?
-    int processed = numberInColumnPlus[i];
-    CoinBigIndex start = startColumnU[i] - processed;
-
-    number += processed;
-    numberInColumn[i] = number;
-    totalElements_ += number;
-    startColumnU[i] = start;
-    //full list
-    numberInColumnPlus[i] = 0;
+#ifndef NDEBUG
+  for ( i = 0; i < numberRows_; i++ ) {
+    assert (permute[i]>=0&&permute[i]<numberRows_);
+    assert (permuteBack[i]>=0&&permuteBack[i]<numberRows_);
   }
-  if ( (messageLevel_ & 8)) {
-    std::cout<<"        length of U "<<totalElements_<<", length of L "<<lengthL_;
-    if (numberDense_)
-      std::cout<<" plus "<<numberDense_*numberDense_<<" from "<<numberDense_<<" dense rows";
-    std::cout<<std::endl;
+#endif
+#if COIN_ONE_ETA_COPY
+  if (!compressDone) {
+#endif
+    // Redo total elements
+    totalElements_=0;
+    for ( i = 0; i < numberColumns_; i++ ) {
+      int number = numberInColumn[i];
+      totalElements_ += number;
+      startColumnU[i] -= number;
+    }
+#if COIN_ONE_ETA_COPY
   }
-  // and add L and dense
-  totalElements_ += numberDense_*numberDense_+lengthL_;
+#endif
   int numberU = 0;
 
   pivotColumnBack_.conditionalNew( maximumRowsExtra_ + 1);
 #ifdef ZEROFAULT
-  memset(pivotColumnBack_.array(),'q',(maximumRowsExtra_+1)*sizeof(int));
+  memset(pivotColumnBack(),'q',(maximumRowsExtra_+1)*sizeof(int));
 #endif
   const int * pivotColumn = pivotColumn_.array();
-  int * pivotColumnBack = pivotColumnBack_.array();
-  for ( i = 0; i < numberColumns_; i++ ) {
-    int iColumn = pivotColumn[i];
-
-    pivotColumnBack[iColumn] = i;
-    if ( iColumn >= 0 ) {
-      if ( !numberInColumnPlus[iColumn] ) {
+  int * pivotColumnB = pivotColumnBack();
+  int *indexColumnU = indexColumnU_.array();
+  CoinBigIndex *startColumn = startColumnU;
+  int *indexRowU = indexRowU_.array();
+  CoinFactorizationDouble *elementU = elementU_.array();
+#if COIN_ONE_ETA_COPY
+  if (!compressDone) {
+#endif
+    for ( i = 0; i < numberColumns_; i++ ) {
+      int iColumn = pivotColumn[i];
+      
+      pivotColumnB[iColumn] = i;
+      if ( iColumn >= 0 ) {
 	//wanted
 	if ( numberU != iColumn ) {
 	  numberInColumnPlus[iColumn] = numberU;
@@ -1533,80 +1679,95 @@ CoinFactorization::cleanup (  )
 	numberU++;
       }
     }
-  }
-  for ( i = 0; i < numberColumns_; i++ ) {
-    int number = numberInColumn[i];	//always 0?
-    int where = numberInColumnPlus[i];
-
-    numberInColumnPlus[i] = -1;
-    CoinBigIndex start = startColumnU[i];
-
-    while ( where >= 0 ) {
-      //put where it should be
-      int numberNext = numberInColumn[where];	//always 0?
-      int whereNext = numberInColumnPlus[where];
-      CoinBigIndex startNext = startColumnU[where];
-
-      numberInColumn[where] = number;
-      numberInColumnPlus[where] = -1;
-      startColumnU[where] = start;
-      number = numberNext;
-      where = whereNext;
-      start = startNext;
-    }				/* endwhile */
-  }
-  //sort - using indexColumn
-  CoinFillN ( indexColumnU_.array(), lastU, -1 );
-  CoinBigIndex k = 0;
-  int *indexColumnU = indexColumnU_.array();
-  CoinBigIndex *startColumn = startColumnU;
-  int *indexRowU = indexRowU_.array();
-  double *elementU = elementU_.array();
-
-  for ( i = numberSlacks_; i < numberRows_; i++ ) {
-    CoinBigIndex start = startColumn[i];
-    CoinBigIndex end = start + numberInColumn[i];
-
-    CoinBigIndex j;
-    for ( j = start; j < end; j++ ) {
-      indexColumnU[j] = k++;
-    }
-  }
-  for ( i = numberSlacks_; i < numberRows_; i++ ) {
-    CoinBigIndex start = startColumn[i];
-    CoinBigIndex end = start + numberInColumn[i];
-
-    CoinBigIndex j;
-    for ( j = start; j < end; j++ ) {
-      CoinBigIndex k = indexColumnU[j];
-      int iRow = indexRowU[j];
-      double element = elementU[j];
-
-      while ( k != -1 ) {
-	CoinBigIndex kNext = indexColumnU[k];
-	int iRowNext = indexRowU[k];
-	double elementNext = elementU[k];
-
-	indexColumnU[k] = -1;
-	indexRowU[k] = iRow;
-	elementU[k] = element;
-	k = kNext;
-	iRow = iRowNext;
-	element = elementNext;
+    for ( i = 0; i < numberColumns_; i++ ) {
+      int number = numberInColumn[i];	//always 0?
+      int where = numberInColumnPlus[i];
+      
+      numberInColumnPlus[i] = -1;
+      CoinBigIndex start = startColumnU[i];
+      
+      while ( where >= 0 ) {
+	//put where it should be
+	int numberNext = numberInColumn[where];	//always 0?
+	int whereNext = numberInColumnPlus[where];
+	CoinBigIndex startNext = startColumnU[where];
+	
+	numberInColumn[where] = number;
+	numberInColumnPlus[where] = -1;
+	startColumnU[where] = start;
+	number = numberNext;
+	where = whereNext;
+	start = startNext;
       }				/* endwhile */
     }
+    //sort - using indexColumn
+    CoinFillN ( indexColumnU_.array(), lastU, -1 );
+    CoinBigIndex k = 0;
+    
+    for ( i = numberSlacks_; i < numberRows_; i++ ) {
+      CoinBigIndex start = startColumn[i];
+      CoinBigIndex end = start + numberInColumn[i];
+      
+      CoinBigIndex j;
+      for ( j = start; j < end; j++ ) {
+	indexColumnU[j] = k++;
+      }
+    }
+    for ( i = numberSlacks_; i < numberRows_; i++ ) {
+      CoinBigIndex start = startColumn[i];
+      CoinBigIndex end = start + numberInColumn[i];
+      
+      CoinBigIndex j;
+      for ( j = start; j < end; j++ ) {
+	CoinBigIndex k = indexColumnU[j];
+	int iRow = indexRowU[j];
+	CoinFactorizationDouble element = elementU[j];
+	
+	while ( k != -1 ) {
+	  CoinBigIndex kNext = indexColumnU[k];
+	  int iRowNext = indexRowU[k];
+	  CoinFactorizationDouble elementNext = elementU[k];
+	  
+	  indexColumnU[k] = -1;
+	  indexRowU[k] = iRow;
+	  elementU[k] = element;
+	  k = kNext;
+	  iRow = iRowNext;
+	  element = elementNext;
+	}				/* endwhile */
+      }
+    }
+    CoinZeroN ( startColumnU, numberSlacks_ );
+    k = 0;
+    for ( i = numberSlacks_; i < numberRows_; i++ ) {
+      startColumnU[i] = k;
+      k += numberInColumn[i];
+    }
+    maximumU_=k;
+#if COIN_ONE_ETA_COPY
+  } else {
+    // U already OK
+    for ( i = 0; i < numberColumns_; i++ ) {
+      int iColumn = pivotColumn[i];
+      pivotColumnB[iColumn] = i;
+    }
+    maximumU_=startColumnU[numberRows_-1]+
+      numberInColumn[numberRows_-1];
+    numberU=numberRows_;
   }
-  CoinZeroN ( startColumnU, numberSlacks_ );
-  k = 0;
-  for ( i = numberSlacks_; i < numberRows_; i++ ) {
-    startColumnU[i] = k;
-    k += numberInColumn[i];
+#endif
+  if ( (messageLevel_ & 8)) {
+    std::cout<<"        length of U "<<totalElements_<<", length of L "<<lengthL_;
+    if (numberDense_)
+      std::cout<<" plus "<<numberDense_*numberDense_<<" from "<<numberDense_<<" dense rows";
+    std::cout<<std::endl;
   }
-  maximumU_=k;
+  // and add L and dense
+  totalElements_ += numberDense_*numberDense_+lengthL_;
   int * nextColumn = nextColumn_.array();
   int * lastColumn = lastColumn_.array();
   // See whether to have extra copy of R
-  if (k>10*numberRows_) {
+  if (maximumU_>10*numberRows_||numberRows_<200) {
     // NO
     numberInColumnPlus_.conditionalDelete() ;
   } else {
@@ -1631,12 +1792,14 @@ CoinFactorization::cleanup (  )
     }
   }
 #endif
+  CoinFactorizationDouble * pivotRegion = pivotRegion_.array();
   for ( i = numberSlacks_; i < numberU; i++ ) {
     CoinBigIndex start = startColumnU[i];
     CoinBigIndex end = start + numberInColumn[i];
 
     totalElements_ += numberInColumn[i];
     CoinBigIndex j;
+
     for ( j = start; j < end; j++ ) {
       int iRow = indexRowU[j];
       iRow = permute[iRow];
@@ -1644,62 +1807,72 @@ CoinFactorization::cleanup (  )
       numberInRow[iRow]++;
     }
   }
-  //space for cross reference
-  convertRowToColumnU_.conditionalNew( lengthAreaU_ );
-  CoinBigIndex *convertRowToColumn = convertRowToColumnU_.array();
-  CoinBigIndex j = 0;
-  CoinBigIndex *startRow = startRowU_.array();
+#if COIN_ONE_ETA_COPY
+  if (numberRows_>=COIN_ONE_ETA_COPY) {
+#endif
+    //space for cross reference
+    convertRowToColumnU_.conditionalNew( lengthAreaU_ );
+    CoinBigIndex *convertRowToColumn = convertRowToColumnU_.array();
+    CoinBigIndex j = 0;
+    CoinBigIndex *startRow = startRowU_.array();
+    
+    int iRow;
+    for ( iRow = 0; iRow < numberRows_; iRow++ ) {
+      startRow[iRow] = j;
+      j += numberInRow[iRow];
+    }
+    CoinBigIndex numberInU = j;
+    
+    CoinZeroN ( numberInRow_.array(), numberRows_ );
+    
+    for ( i = numberSlacks_; i < numberRows_; i++ ) {
+      CoinBigIndex start = startColumnU[i];
+      CoinBigIndex end = start + numberInColumn[i];
+      
+      CoinFactorizationDouble pivotValue = pivotRegion[i];
 
-  int iRow;
-  for ( iRow = 0; iRow < numberRows_; iRow++ ) {
-    startRow[iRow] = j;
-    j += numberInRow[iRow];
-  }
-  CoinBigIndex numberInU = j;
-
-  CoinZeroN ( numberInRow_.array(), numberRows_ );
-  CoinBigIndex lowCount = 0;
-  CoinBigIndex highCount = numberInU;
-  int lowC = 0;
-  double * pivotRegion = pivotRegion_.array();
-  int highC = numberRows_ - numberSlacks_;
-
-  for ( i = numberSlacks_; i < numberRows_; i++ ) {
-    CoinBigIndex start = startColumnU[i];
-    CoinBigIndex end = start + numberInColumn[i];
-
-    lowCount += numberInColumn[i];
-    highCount -= numberInColumn[i];
-    lowC++;
-    highC--;
-    double pivotValue = pivotRegion[i];
-
-    CoinBigIndex j;
-    for ( j = start; j < end; j++ ) {
-      int iRow = indexRowU[j];
-      int iLook = numberInRow[iRow];
-
-      numberInRow[iRow] = iLook + 1;
-      CoinBigIndex k = startRow[iRow] + iLook;
-
-      indexColumnU[k] = i;
-      convertRowToColumn[k] = j;
-      //multiply by pivot
-      elementU[j] *= pivotValue;
+      CoinBigIndex j;
+      for ( j = start; j < end; j++ ) {
+	int iRow = indexRowU[j];
+	int iLook = numberInRow[iRow];
+	
+	numberInRow[iRow] = iLook + 1;
+	CoinBigIndex k = startRow[iRow] + iLook;
+	
+	indexColumnU[k] = i;
+	convertRowToColumn[k] = j;
+	//multiply by pivot
+	elementU[j] *= pivotValue;
+      }
+    }
+    int * nextRow = nextRow_.array();
+    int * lastRow = lastRow_.array();
+    for ( j = 0; j < numberRows_; j++ ) {
+      lastRow[j] = j - 1;
+      nextRow[j] = j + 1;
+    }
+    nextRow[numberRows_ - 1] = maximumRowsExtra_;
+    lastRow[maximumRowsExtra_] = numberRows_ - 1;
+    nextRow[maximumRowsExtra_] = 0;
+    lastRow[0] = maximumRowsExtra_;
+    startRow[maximumRowsExtra_] = numberInU;
+#if COIN_ONE_ETA_COPY
+  } else {
+    // no row copy
+    for ( i = numberSlacks_; i < numberU; i++ ) {
+      CoinBigIndex start = startColumnU[i];
+      CoinBigIndex end = start + numberInColumn[i];
+      
+      CoinBigIndex j;
+      CoinFactorizationDouble pivotValue = pivotRegion[i];
+      
+      for ( j = start; j < end; j++ ) {
+	//multiply by pivot
+	elementU[j] *= pivotValue;
+      }
     }
   }
-  int * nextRow = nextRow_.array();
-  int * lastRow = lastRow_.array();
-  for ( j = 0; j < numberRows_; j++ ) {
-    lastRow[j] = j - 1;
-    nextRow[j] = j + 1;
-  }
-  nextRow[numberRows_ - 1] = maximumRowsExtra_;
-  lastRow[maximumRowsExtra_] = numberRows_ - 1;
-  nextRow[maximumRowsExtra_] = 0;
-  lastRow[0] = maximumRowsExtra_;
-  startRow[maximumRowsExtra_] = numberInU;
-  int * pivotRowL = pivotRowL_.array();
+#endif
 
   int firstReal = numberRows_;
 
@@ -1710,25 +1883,20 @@ CoinFactorization::cleanup (  )
     CoinBigIndex end = startColumnL[i + 1];
 
     totalElements_ += end - start;
-    int pivotRow = pivotRowL[i];
-
-    pivotRow = permute[pivotRow];
-    pivotRowL[i] = pivotRow;
     if ( end > start ) {
       firstReal = i;
       CoinBigIndex j;
       for ( j = start; j < end; j++ ) {
 	int iRow = indexRowL[j];
-
 	iRow = permute[iRow];
+	assert (iRow>firstReal);
 	indexRowL[j] = iRow;
       }
     }
   }
   baseL_ = firstReal;
-  numberL_ = numberGoodL_ - firstReal;
+  numberL_ -= firstReal;
   factorElements_ = totalElements_;
-  pivotRowL[numberGoodL_] = numberRows_;	//so loop will be clean
   //can delete pivotRowL_ as not used
   pivotRowL_.conditionalDelete() ;
   //use L for R if room
@@ -1896,20 +2064,18 @@ CoinFactorization::pivotOneOtherRow ( int pivotRow,
   }
   //l+=currentAreaL_->elementByColumn-elementL_;
   //CoinBigIndex lSave=l;
-  int * pivotRowL = pivotRowL_.array();
-  pivotRowL[numberGoodL_] = pivotRow;
   CoinBigIndex * startColumnL = startColumnL_.array();
-  double * elementL = elementL_.array();
+  CoinFactorizationDouble * elementL = elementL_.array();
   int * indexRowL = indexRowL_.array();
   startColumnL[numberGoodL_] = l;	//for luck and first time
   numberGoodL_++;
   startColumnL[numberGoodL_] = l + 1;
   lengthL_++;
-  double pivotElement;
-  double otherMultiplier;
+  CoinFactorizationDouble pivotElement;
+  CoinFactorizationDouble otherMultiplier;
   int otherRow;
   int * saveColumn = saveColumn_.array();
-  double *elementU = elementU_.array();
+  CoinFactorizationDouble *elementU = elementU_.array();
   int * indexRowU = indexRowU_.array();
 
   if ( indexRowU[startColumn] == pivotRow ) {
@@ -1922,9 +2088,9 @@ CoinFactorization::pivotOneOtherRow ( int pivotRow,
     otherRow = indexRowU[startColumn];
   }
   int numberSave = numberInRow[otherRow];
-  double pivotMultiplier = 1.0 / pivotElement;
+  CoinFactorizationDouble pivotMultiplier = 1.0 / pivotElement;
 
-  double * pivotRegion = pivotRegion_.array();
+  CoinFactorizationDouble * pivotRegion = pivotRegion_.array();
   pivotRegion[numberGoodU_] = pivotMultiplier;
   numberInColumn[pivotColumn] = 0;
   otherMultiplier = otherMultiplier * pivotMultiplier;
@@ -1957,16 +2123,16 @@ CoinFactorization::pivotOneOtherRow ( int pivotRow,
       CoinBigIndex startColumn = startColumnU[iColumn];
       CoinBigIndex endColumn = startColumn + numberInColumn[iColumn];
       int iRow = indexRowU[startColumn];
-      double value = elementU[startColumn];
+      CoinFactorizationDouble value = elementU[startColumn];
       double largest;
       bool foundOther = false;
 
       //leave room for pivot
       CoinBigIndex put = startColumn + 1;
       CoinBigIndex positionLargest = -1;
-      double thisPivotValue = 0.0;
-      double otherElement = 0.0;
-      double nextValue = elementU[put];;
+      CoinFactorizationDouble thisPivotValue = 0.0;
+      CoinFactorizationDouble otherElement = 0.0;
+      CoinFactorizationDouble nextValue = elementU[put];;
       int nextIRow = indexRowU[put];
 
       //compress column and find largest not updated
@@ -2188,7 +2354,7 @@ CoinFactorization::setPersistenceFlag(int flag)
   numberInRow_.setPersistence(flag,maximumRowsExtra_+1);
   numberInColumn_.setPersistence(flag,maximumColumnsExtra_+1);
   numberInColumnPlus_.setPersistence(flag,maximumColumnsExtra_+1);
-  firstCount_.setPersistence(flag,biggerDimension_+2);
+  firstCount_.setPersistence(flag,CoinMax(biggerDimension_+2,maximumRowsExtra_+1));
   nextCount_.setPersistence(flag,numberRows_+numberColumns_);
   lastCount_.setPersistence(flag,numberRows_+numberColumns_);
   nextColumn_.setPersistence(flag,maximumColumnsExtra_+1);
