@@ -1,3 +1,4 @@
+/* $Id: CoinPresolveMatrix.hpp 1269 2010-04-02 17:15:06Z forrest $ */
 // Copyright (C) 2002, International Business Machines
 // Corporation and others.  All Rights Reserved.
 #ifndef CoinPresolveMatrix_H
@@ -54,10 +55,10 @@ inline void DIE(const char *s)	{ std::cout<<s; abort(); }
 
 #else
 
-#define PRESOLVEASSERT(x) 
-#define	PRESOLVE_STMT(s)
+#define PRESOLVEASSERT(x) {} 
+#define	PRESOLVE_STMT(s) {}
 
-inline void DIE(const char *s)	{}
+inline void DIE(const char *)	{}
 
 #endif
 
@@ -154,6 +155,9 @@ class CoinPresolveAction
     one passed as the parameter.
   */
   CoinPresolveAction(const CoinPresolveAction *next) : next(next) {}
+  /// modify next (when building rather than passing)
+  inline void setNext(const CoinPresolveAction *nextAction)
+  { next = nextAction;}
 
   /*! \brief A name for debug printing.
 
@@ -300,8 +304,8 @@ class CoinPrePostsolveMatrix
   inline void setRowStatus(int sequence, Status status)
   {
     unsigned char & st_byte = rowstat_[sequence];
-    st_byte &= ~7;
-    st_byte |= status;
+    st_byte = static_cast<unsigned char>(st_byte & (~7)) ;
+    st_byte = static_cast<unsigned char>(st_byte | status) ;
   }
   /// Get row status
   inline Status getRowStatus(int sequence) const
@@ -313,8 +317,8 @@ class CoinPrePostsolveMatrix
   inline void setColumnStatus(int sequence, Status status)
   {
     unsigned char & st_byte = colstat_[sequence];
-    st_byte &= ~7;
-    st_byte |= status;
+    st_byte = static_cast<unsigned char>(st_byte & (~7)) ;
+    st_byte = static_cast<unsigned char>(st_byte | status) ;
 
 #   ifdef PRESOLVE_DEBUG
     switch (status)
@@ -806,17 +810,16 @@ class CoinPresolveMatrix : public CoinPrePostsolveMatrix
     See OSI code for the definition.
   */
   CoinPresolveMatrix(int ncols0,
-		    double maxmin,
-		    // end prepost members
-
-		    OsiSolverInterface * si,
-
-		    // rowrep
-		    int nrows,
-		    CoinBigIndex nelems,
-		 bool doStatus,
-		 double nonLinearVariable,
-                     const char * prohibited);
+		     double maxmin,
+		     // end prepost members
+		     OsiSolverInterface * si,
+		     // rowrep
+		     int nrows,
+		     CoinBigIndex nelems,
+		     bool doStatus,
+		     double nonLinearVariable,
+                     const char * prohibited,
+		     const char * rowProhibited=NULL);
 
   /*! \brief Update the model held by a generic OSI */
   void update_model(OsiSolverInterface * si,
@@ -857,7 +860,7 @@ class CoinPresolveMatrix : public CoinPrePostsolveMatrix
   */
   inline void setVariableType(int i, int variableType)
   { if (integerType_ == 0) integerType_ = new unsigned char [ncols0_] ;
-    integerType_[i] = variableType ; }
+    integerType_[i] = static_cast<unsigned char>(variableType) ; }
 
   /*! \brief Set variable type information for all variables
   
@@ -945,7 +948,7 @@ class CoinPresolveMatrix : public CoinPrePostsolveMatrix
   #endif
     if (change_amount)
       PRESOLVE_STMT(printf("changing bias by %g to %g\n",
-		      change_amount, dobias_));  
+			    change_amount, dobias_));
   }
 
   /*! \name Row-major representation
@@ -1077,6 +1080,9 @@ class CoinPresolveMatrix : public CoinPrePostsolveMatrix
       1 set if allow duplicate column tests for integer variables
       2 set to allow code to try and fix infeasibilities
       4 set to inhibit x+y+z=1 mods
+      8 not used
+      16 set to allow stuff which won't unroll easily 
+      0x80000000 set by presolve to say dupcol_action compressed columns
   */
   int presolveOptions_;
   /*! Flag to say if any rows or columns are marked as prohibited
@@ -1085,6 +1091,24 @@ class CoinPresolveMatrix : public CoinPrePostsolveMatrix
     various \c set*Prohibited routines.
   */
   bool anyProhibited_;
+  /// Useful int array 3* number rows
+  int * usefulRowInt_;
+  /// Useful double array number rows
+  double * usefulRowDouble_;
+  /// Useful int array 2* number columns
+  int * usefulColumnInt_;
+  /// Useful double array number columns
+  double * usefulColumnDouble_;
+  /// Array of random numbers (max row,column)
+  double * randomNumber_;
+  /// Array giving number of infinite ups on a row
+  int * infiniteUp_;
+  /// Array giving sum of non-infinite ups on a row
+  double * sumUp_;
+  /// Array giving number of infinite downs on a row
+  int * infiniteDown_;
+  /// Array giving sum of non-infinite downs on a row
+  double * sumDown_;
   //@}
 
   /*! \name Functions to manipulate row and column processing status */
@@ -1114,16 +1138,16 @@ class CoinPresolveMatrix : public CoinPrePostsolveMatrix
   }
   /// Mark column as not changed
   inline void unsetColChanged(int i) {
-    colChanged_[i]  &= ~1;;
+    colChanged_[i] = static_cast<unsigned char>(colChanged_[i] & (~1)) ;
   }
   /// Mark column as changed.
   inline void setColChanged(int i) {
-    colChanged_[i] |= 1;
+    colChanged_[i] = static_cast<unsigned char>(colChanged_[i] | (1)) ;
   }
   /// Mark column as changed and add to list of columns to process next
   inline void addCol(int i) {
     if ((colChanged_[i]&1)==0) {
-      colChanged_[i] |= 1;
+      colChanged_[i] = static_cast<unsigned char>(colChanged_[i] | (1)) ;
       nextColsToDo_[numberNextColsToDo_++] = i;
     }
   }
@@ -1145,7 +1169,7 @@ class CoinPresolveMatrix : public CoinPrePostsolveMatrix
   }
   /// Mark column as ineligible for preprocessing
   inline void setColProhibited(int i) {
-    colChanged_[i] |= 2;
+    colChanged_[i] = static_cast<unsigned char>(colChanged_[i] | (2)) ;
   }
   /*! \brief Test if column is marked as used
   
@@ -1157,11 +1181,11 @@ class CoinPresolveMatrix : public CoinPrePostsolveMatrix
   }
   /// Mark column as used
   inline void setColUsed(int i) {
-    colChanged_[i] |= 4;
+    colChanged_[i] = static_cast<unsigned char>(colChanged_[i] | (4)) ;
   }
   /// Mark column as unused
   inline void unsetColUsed(int i) {
-    colChanged_[i]  &= ~4;;
+    colChanged_[i] = static_cast<unsigned char>(colChanged_[i] & (~4)) ;
   }
 
   /*! \brief Initialise the row ToDo lists
@@ -1188,16 +1212,16 @@ class CoinPresolveMatrix : public CoinPrePostsolveMatrix
   }
   /// Mark row as not changed
   inline void unsetRowChanged(int i) {
-    rowChanged_[i]  &= ~1;;
+    rowChanged_[i] = static_cast<unsigned char>(rowChanged_[i] & (~1)) ;
   }
   /// Mark row as changed
   inline void setRowChanged(int i) {
-    rowChanged_[i] |= 1;
+    rowChanged_[i] = static_cast<unsigned char>(rowChanged_[i] | (1)) ;
   }
   /// Mark row as changed and add to list of rows to process next
   inline void addRow(int i) {
     if ((rowChanged_[i]&1)==0) {
-      rowChanged_[i] |= 1;
+      rowChanged_[i] = static_cast<unsigned char>(rowChanged_[i] | (1)) ;
       nextRowsToDo_[numberNextRowsToDo_++] = i;
     }
   }
@@ -1219,7 +1243,7 @@ class CoinPresolveMatrix : public CoinPrePostsolveMatrix
   }
   /// Mark row as ineligible for preprocessing
   inline void setRowProhibited(int i) {
-    rowChanged_[i] |= 2;
+    rowChanged_[i] = static_cast<unsigned char>(rowChanged_[i] | (2)) ;
   }
   /*! \brief Test if row is marked as used
 
@@ -1231,11 +1255,11 @@ class CoinPresolveMatrix : public CoinPrePostsolveMatrix
   }
   /// Mark row as used
   inline void setRowUsed(int i) {
-    rowChanged_[i] |= 4;
+    rowChanged_[i] = static_cast<unsigned char>(rowChanged_[i] | (4)) ;
   }
   /// Mark row as unused
   inline void unsetRowUsed(int i) {
-    rowChanged_[i]  &= ~4;;
+    rowChanged_[i] = static_cast<unsigned char>(rowChanged_[i] & (~4)) ;
   }
 
 
@@ -1245,6 +1269,13 @@ class CoinPresolveMatrix : public CoinPrePostsolveMatrix
   /// Set a flag for presence of prohibited rows or columns
   inline void setAnyProhibited(bool val = true)
   { anyProhibited_ = val ; }
+  /** Recompute ups and downs for a row (nonzero if infeasible).
+      If iRow -1 then recompute all */
+  int recomputeSums(int iRow);
+  /// Initialize random numbers etc (nonzero if infeasible)
+  int initializeStuff();
+  /// Delete useful arrays 
+  void deleteStuff();
   //@}
 
 };
@@ -1394,7 +1425,7 @@ class CoinPostsolveMatrix : public CoinPrePostsolveMatrix
     \brief Initialise linked list for major vector order in bulk storage
 */
 
-void presolve_make_memlists(CoinBigIndex *starts, int *lengths,
+void presolve_make_memlists(/*CoinBigIndex *starts,*/ int *lengths,
 			    presolvehlink *link, int n);
 
 /*! \relates CoinPrePostsolveMatrix
@@ -1439,8 +1470,22 @@ inline bool presolve_expand_row(CoinBigIndex *mrstrt, double *rowels,
     used directly or via the inline wrappers presolve_find_row and
     presolve_find_col.
 */
-CoinBigIndex presolve_find_minor(int tgt, CoinBigIndex ks, CoinBigIndex ke,
-				 const int *minndxs);
+inline CoinBigIndex presolve_find_minor(int tgt, CoinBigIndex ks, CoinBigIndex ke,
+				 const int *minndxs)
+{ CoinBigIndex k ;
+  for (k = ks ; k < ke ; k++)
+#ifndef NDEBUG
+  { if (minndxs[k] == tgt)
+      return (k) ; }
+  DIE("FIND_MINOR") ;
+
+  abort () ; return -1;
+#else
+  { if (minndxs[k] == tgt)
+      break ; }
+  return (k) ;
+#endif
+}
 
 /*! \relates CoinPrePostsolveMatrix
     \brief Find position of a row in a column in a column-major matrix.
@@ -1551,13 +1596,39 @@ inline CoinBigIndex presolve_find_row3(int row, CoinBigIndex kcs, int collen,
    majlens) is decremented.  Loose packing is maintained by swapping the last
    entry in the row into the position occupied by the deleted entry.
 */
-void presolve_delete_from_major(int majndx, int minndx,
+inline void presolve_delete_from_major(int majndx, int minndx,
 				const CoinBigIndex *majstrts,
-				int *majlens, int *minndxs, double *els) ;
+				int *majlens, int *minndxs, double *els) 
+{ CoinBigIndex ks = majstrts[majndx] ;
+  CoinBigIndex ke = ks + majlens[majndx] ;
+
+  CoinBigIndex kmi = presolve_find_minor(minndx,ks,ke,minndxs) ;
+
+  minndxs[kmi] = minndxs[ke-1] ;
+  els[kmi] = els[ke-1] ;
+  majlens[majndx]-- ;
+  
+  return ; }
 // Delete all marked from major (and zero marked)
-void presolve_delete_many_from_major(int majndx, char * marked,
+inline void presolve_delete_many_from_major(int majndx, char * marked,
 				const CoinBigIndex *majstrts,
-				int *majlens, int *minndxs, double *els) ;
+				int *majlens, int *minndxs, double *els) 
+{ 
+  CoinBigIndex ks = majstrts[majndx] ;
+  CoinBigIndex ke = ks + majlens[majndx] ;
+  CoinBigIndex put=ks;
+  for (CoinBigIndex k=ks;k<ke;k++) {
+    int iMinor = minndxs[k];
+    if (!marked[iMinor]) {
+      minndxs[put]=iMinor;
+      els[put++]=els[k];
+    } else {
+      marked[iMinor]=0;
+    }
+  } 
+  majlens[majndx] = put-ks ;
+  return ;
+}
 
 /*! \relates CoinPrePostsolveMatrix
     \brief Delete the entry for row \p row from column \p col in a
@@ -1601,7 +1672,7 @@ inline void presolve_delete_from_row(int row, int col,
 */
 void presolve_delete_from_major2 (int majndx, int minndx,
 				  CoinBigIndex *majstrts, int *majlens,
-				  int *minndxs, double *els, int *majlinks,
+				  int *minndxs, /*double *els,*/ int *majlinks,
 				   CoinBigIndex *free_listp) ;
 
 /*! \relates CoinPostsolveMatrix
@@ -1616,9 +1687,9 @@ void presolve_delete_from_major2 (int majndx, int minndx,
 */
 inline void presolve_delete_from_col2(int row, int col, CoinBigIndex *mcstrt,
 				      int *hincol, int *hrow,
-				      double *colels, int *clinks,
+				      /*double *colels,*/ int *clinks,
 				      CoinBigIndex *free_listp)
-{ presolve_delete_from_major2(col,row,mcstrt,hincol,hrow,colels,clinks,
+{ presolve_delete_from_major2(col,row,mcstrt,hincol,hrow,/*colels,*/clinks,
 			      free_listp) ; }
 
 //@}
@@ -1642,7 +1713,8 @@ inline void presolve_delete_from_col2(int row, int col, CoinBigIndex *mcstrt,
 */
 double *presolve_dupmajor(const double *elems, const int *indices,
 			  int length, CoinBigIndex offset, int tgt = -1);
-
+/// Initialize an array with random numbers
+void coin_init_random_vec(double *work, int n);
 //@}
 
 
