@@ -1,3 +1,4 @@
+/* $Id: CoinWarmStartBasis.cpp 1291 2010-05-13 14:37:11Z forrest $ */
 // Copyright (C) 2000, International Business Machines
 // Corporation and others.  All Rights Reserved.
 #if defined(_MSC_VER)
@@ -274,7 +275,7 @@ CoinWarmStartBasis::deleteRows (int rawTgtCnt, const int *rawTgts)
     int *endUnique ;
     std::sort(first,last) ;
     endUnique = std::unique(first,last) ;
-    int tgtCnt = endUnique-first ;
+    int tgtCnt = static_cast<int>(endUnique-first) ;
     compressRows(tgtCnt,tgts) ;
     delete [] tgts ;
   }
@@ -373,6 +374,8 @@ void CoinWarmStartBasis::mergeBasis (const CoinWarmStartBasis *src,
     { int srcNdx = (*xferSpec).first ;
       int tgtNdx = (*xferSpec).second ;
       int runLen = (*xferSpec).third ;
+      // adjust in case Pierre has changed size
+      runLen=CoinMin(runLen,srcRows-srcNdx);
       assert(srcNdx >= 0 && srcNdx+runLen <= srcRows) ;
       assert(tgtNdx >= 0 && tgtNdx+runLen <= getNumArtificial()) ;
       for (int i = 0 ; i < runLen ; i++)
@@ -392,6 +395,7 @@ CoinWarmStartBasis::print() const
     if (status==CoinWarmStartBasis::basic) 
       numberBasic++;
   }
+  int numberStructBasic = numberBasic;
   for (i=0;i<numArtificial_;i++) {
     Status status = getArtifStatus(i);
     if (status==CoinWarmStartBasis::basic) 
@@ -399,7 +403,8 @@ CoinWarmStartBasis::print() const
   }
   std::cout<<"Basis "<<this<<" has "<<numArtificial_<<" rows and "
 	   <<numStructural_<<" columns, "
-	   <<numberBasic<<" basic"<<std::endl;
+	   <<numberBasic<<" basic, of which "<<numberStructBasic
+	   <<" were columns"<<std::endl;
   std::cout<<"Rows:"<<std::endl;
   char type[]={'F','B','U','L'};
 
@@ -461,6 +466,50 @@ CoinWarmStartBasis::fullBasis() const
 #endif
   return numberBasic==numArtificial_;
 }
+// Returns true if full basis and fixes up (for debug)
+bool 
+CoinWarmStartBasis::fixFullBasis() 
+{
+  int i ;
+  int numberBasic=0;
+  for (i=0;i<numStructural_;i++) {
+    Status status = getStructStatus(i);
+    if (status==CoinWarmStartBasis::basic) 
+      numberBasic++;
+  }
+  for (i=0;i<numArtificial_;i++) {
+    Status status = getArtifStatus(i);
+    if (status==CoinWarmStartBasis::basic) 
+      numberBasic++;
+  }
+#ifdef COIN_DEVELOP
+  if (numberBasic!=numArtificial_)
+    printf("mismatch - basis has %d rows, %d basic\n",
+	   numArtificial_,numberBasic);
+#endif
+  bool returnCode = (numberBasic==numArtificial_);
+  if (numberBasic>numArtificial_) {
+    for (i=0;i<numStructural_;i++) {
+      Status status = getStructStatus(i);
+      if (status==CoinWarmStartBasis::basic) 
+	setStructStatus(i,atLowerBound);
+	numberBasic--;
+	if (numberBasic==numArtificial_)
+	  break;
+    }
+  } else if (numberBasic<numArtificial_) {
+    for (i=0;i<numArtificial_;i++) {
+      Status status = getArtifStatus(i);
+      if (status!=CoinWarmStartBasis::basic) {
+	setArtifStatus(i,basic);
+	numberBasic++;
+	if (numberBasic==numArtificial_)
+	  break;
+      }
+    }
+  }
+  return returnCode;
+}
 /*
   Generate a diff that'll convert oldCWS into the basis pointed to by this.
 
@@ -472,15 +521,17 @@ CoinWarmStartBasis::fullBasis() const
 
 CoinWarmStartDiff*
 CoinWarmStartBasis::generateDiff (const CoinWarmStart *const oldCWS) const
-{ 
+{
 /*
   Make sure the parameter is CoinWarmStartBasis or derived class.
 */
   const CoinWarmStartBasis *oldBasis =
       dynamic_cast<const CoinWarmStartBasis *>(oldCWS) ;
+#ifndef NDEBUG 
   if (!oldBasis)
   { throw CoinError("Old basis not derived from CoinWarmStartBasis.",
 		    "generateDiff","CoinWarmStartBasis") ; }
+#endif
   const CoinWarmStartBasis *newBasis = this ;
 /*
   Make sure newBasis is equal or bigger than oldBasis. Calculate the worst case
@@ -554,7 +605,7 @@ CoinWarmStartBasis::generateDiff (const CoinWarmStart *const oldCWS) const
 */
   delete[] diffNdx ;
 
-  return (dynamic_cast<CoinWarmStartDiff *>(diff)) ; }
+  return (static_cast<CoinWarmStartDiff *>(diff)) ; }
 
 
 /*
@@ -568,9 +619,11 @@ void CoinWarmStartBasis::applyDiff (const CoinWarmStartDiff *const cwsdDiff)
 */
   const CoinWarmStartBasisDiff *diff =
     dynamic_cast<const CoinWarmStartBasisDiff *>(cwsdDiff) ;
+#ifndef NDEBUG 
   if (!diff)
   { throw CoinError("Diff not derived from CoinWarmStartBasisDiff.",
 		    "applyDiff","CoinWarmStartBasis") ; }
+#endif
 /*
   Application is by straighforward replacement of words in the status arrays.
   Index entries for logicals (aka artificials) are tagged with 0x80000000.
@@ -594,14 +647,13 @@ void CoinWarmStartBasis::applyDiff (const CoinWarmStartDiff *const cwsdDiff)
   } else {
     // just replace
     const unsigned int * diffA = diff->difference_ -1;
-    const int artifCnt = (int) diffA[0];
+    const int artifCnt = static_cast<int> (diffA[0]);
     const int structCnt = -numberChanges;
     int sizeArtif = (artifCnt+15)>>4 ;
     int sizeStruct = (structCnt+15)>>4 ;
     CoinMemcpyN(diffA+1,sizeStruct,structStatus);
     CoinMemcpyN(diffA+1+sizeStruct,sizeArtif,artifStatus);
   }
-
   return ; }
 
 
@@ -638,9 +690,9 @@ CoinWarmStartBasisDiff::CoinWarmStartBasisDiff (const CoinWarmStartBasis * rhs)
   difference_ = new unsigned int [maxBasisLength+1];
   difference_[0]=artifCnt;
   difference_++;
-  CoinMemcpyN((const unsigned int *) rhs->getStructuralStatus(),sizeStruct,
+  CoinMemcpyN(reinterpret_cast<const unsigned int *> (rhs->getStructuralStatus()),sizeStruct,
 	      difference_);
-  CoinMemcpyN((const unsigned int *) rhs->getArtificialStatus(),sizeArtif,
+  CoinMemcpyN(reinterpret_cast<const unsigned int *> (rhs->getArtificialStatus()),sizeArtif,
 	      difference_+sizeStruct);
 }
 
@@ -656,7 +708,7 @@ CoinWarmStartBasisDiff::CoinWarmStartBasisDiff
     { difference_ = CoinCopyOfArray(rhs.difference_,2*sze_); }
   else if (sze_<0) {
     const unsigned int * diff = rhs.difference_ -1;
-    const int artifCnt = (int) diff[0];
+    const int artifCnt = static_cast<int> (diff[0]);
     const int structCnt = -sze_;
     int sizeArtif = (artifCnt+15)>>4 ;
     int sizeStruct = (structCnt+15)>>4 ;
@@ -686,7 +738,7 @@ CoinWarmStartBasisDiff::operator= (const CoinWarmStartBasisDiff &rhs)
       { difference_ = CoinCopyOfArray(rhs.difference_,2*sze_); }
     else if (sze_<0) {
       const unsigned int * diff = rhs.difference_ -1;
-      const int artifCnt = (int) diff[0];
+      const int artifCnt = static_cast<int> (diff[0]);
       const int structCnt = -sze_;
       int sizeArtif = (artifCnt+15)>>4 ;
       int sizeStruct = (structCnt+15)>>4 ;
